Class: Roda::Component

Inherits:
Object
  • Object
show all
Defined in:
lib/roda/component.rb,
lib/roda/component/dom.rb,
lib/roda/component/faye.rb,
lib/roda/component/faye.rb,
lib/roda/component/form.rb,
lib/roda/component/events.rb,
lib/roda/component/version.rb,
lib/roda/component/instance.rb,
lib/roda/component/form/validations.rb

Defined Under Namespace

Classes: DOM, Events, Faye, Form, Instance

Constant Summary collapse

VERSION =
"0.1.21"

Class Attribute Summary collapse

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(scope = false) ⇒ Component

Returns a new instance of Component.



53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
# File 'lib/roda/component.rb', line 53

def initialize(scope = false)
  @scope   = scope

  if client?
    $faye.subscribe "/components/#{self.class._name}" do |msg|
      msg = Native(msg)

      trigger :"#{msg[:type]}", msg['local'], msg unless $faye.public_id == msg[:public_id]
    end

    $faye.on 'transport:up' do
      $faye.online = true

      if $faye.disconnected
        trigger :reconnect
      else
        trigger :connect
      end
    end

    $faye.on 'transport:down' do
      $faye.disconnected = true
      $faye.online       = false
      trigger :disconnect
    end
  end
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(method, *args, &block) ⇒ Object



310
311
312
313
314
315
316
# File 'lib/roda/component.rb', line 310

def method_missing method, *args, &block
  if server? && scope.respond_to?(method, true)
    scope.send method, *args, &block
  else
    super
  end
end

Class Attribute Details

._nameObject

Returns the value of attribute _name.



82
83
84
# File 'lib/roda/component.rb', line 82

def _name
  @_name
end

Instance Attribute Details

#cacheObject

Returns the value of attribute cache.



51
52
53
# File 'lib/roda/component.rb', line 51

def cache
  @cache
end

#scopeObject

Returns the value of attribute scope.



51
52
53
# File 'lib/roda/component.rb', line 51

def scope
  @scope
end

Class Method Details

.add_tmplObject

We need to save the oga dom and the raw html. the reason we ave the raw html is so that we can use it client side.



242
243
244
245
246
# File 'lib/roda/component.rb', line 242

def tmpl name, dom, remove = true
  cache[:tmpl][name] = { dom: remove ? dom.remove : dom }
  cache[:tmpl][name][:html] = cache[:tmpl][name][:dom].to_html
  cache[:tmpl][name]
end

.appObject

roda app method



231
232
233
# File 'lib/roda/component.rb', line 231

def app
  @_app ||= {}
end

.cacheObject

cache for class



215
216
217
218
219
220
221
222
223
# File 'lib/roda/component.rb', line 215

def cache
  unless @_cache
    @_cache ||= Roda::RodaCache.new
    @_cache[:tmpl] = {}
    @_cache[:server_methods] = []
  end

  @_cache
end

.comp_html(_html, &block) ⇒ Object

The html source



167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
# File 'lib/roda/component.rb', line 167

def comp_html _html, &block
  if server?
    if _html.is_a? String
      if _html[%r{\A./}]
        cache[:html] = File.read _html
      else
        cache[:html] = File.read "#{component_opts[:path]}/#{_html}"
      end
    else
      cache[:html] = yield
    end

    cache[:dom] = DOM.new(cache[:html])
  end
end

.comp_name(_name = nil) ⇒ Object



156
157
158
159
160
161
162
163
164
# File 'lib/roda/component.rb', line 156

def comp_name(_name = nil)
  return name_original unless _name

  @_name = _name.to_s

  if server?
    component_opts[:class_name][@_name] = self.to_s
  end
end

.comp_setup(&block) ⇒ Object

setup your dom



198
199
200
# File 'lib/roda/component.rb', line 198

def comp_setup &block
  block.call cache[:dom] if server?
end

.component_optsObject

shortcut to comp opts



246
247
248
249
250
251
252
# File 'lib/roda/component.rb', line 246

def component_opts
  if client?
    $component_opts
  else
    super
  end
end

.eventsObject



202
203
204
# File 'lib/roda/component.rb', line 202

def events
  @_events ||= Events.new self, component_opts, false
end

.HTML(raw_html) ⇒ Object



183
184
185
186
187
188
189
190
191
192
193
194
195
# File 'lib/roda/component.rb', line 183

def HTML raw_html
  if raw_html[/\A<!DOCTYPE/] || raw_html[/\A<html/]
    Nokogiri::HTML(raw_html)
  else
    parsed_html = Nokogiri::HTML.fragment(raw_html)

    if parsed_html.children.length >= 1
      parsed_html.children.first
    else
      parsed_html
    end
  end
end

.method_missing(method, *args, &block) ⇒ Object



254
255
256
257
258
259
260
# File 'lib/roda/component.rb', line 254

def method_missing method, *args, &block
  if server? && app.respond_to?(method, true)
    app.send method, *args, &block
  else
    super
  end
end

.on(*args, &block) ⇒ Object



206
207
208
209
210
211
212
# File 'lib/roda/component.rb', line 206

def on *args, &block
  if args.first.to_s != 'server'
    events.on(*args, &block)
  else
    on_server(&block)
  end
end

.on_server(&block) ⇒ Object



92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
# File 'lib/roda/component.rb', line 92

def on_server &block
  if server?
    m = Module.new(&block)

    yield

    m.public_instance_methods(false).each do |meth|
      alias_method :"original_#{meth}", :"#{meth}"
      define_method "#{meth}" do |*args, &blk|
        if blk
          blk.call send("original_#{meth}", *args)
        else
          send("original_#{meth}", *args)
        end
      end
    end
  else
    m = Module.new(&block)

    m.public_instance_methods(false).each do |meth|
      define_method "#{meth}" do |*args, &blk|
        name     = self.class._name
        # event_id = "comp-event-#{$faye.generate_id}"

        HTTP.post("/components/#{name}/call/#{meth}",
          headers: {
            'X-CSRF-TOKEN' => Element.find('meta[name=_csrf]').attr('content'),
            'X-RODA-COMPONENT-ON-SERVER' => true
          },
          payload: args) do |response|

            # We set the new csrf token
            xhr  = Native(response.xhr)
            csrf = xhr.getResponseHeader('NEW-CSRF')
            Element.find('meta[name=_csrf]').attr 'content', csrf
            ###########################

            blk.call Native(response.body), response
        end

        # Element['body'].on event_id do |event, local, data|
        #   blk.call local, event, data
        # end
        #
        # $faye.publish("/components/outgoing/#{$faye.private_id}/#{$faye.public_id}", {
        #   name: name,
        #   type: 'event',
        #   event_type: 'call',
        #   event_method: meth,
        #   event_id: event_id,
        #   local: args.first || nil
        # })


        true
      end
    end

    include m
  end
end

.set_app(app) ⇒ Object

set the current roda app



226
227
228
# File 'lib/roda/component.rb', line 226

def set_app app
  @_app = app.respond_to?(:new) ? app.new : app
end

.set_tmplObject

We need to save the oga dom and the raw html. the reason we ave the raw html is so that we can use it client side.



243
244
245
246
247
# File 'lib/roda/component.rb', line 243

def tmpl name, dom, remove = true
  cache[:tmpl][name] = { dom: remove ? dom.remove : dom }
  cache[:tmpl][name][:html] = cache[:tmpl][name][:dom].to_html
  cache[:tmpl][name]
end

.tmpl(name, dom, remove = true) ⇒ Object

We need to save the oga dom and the raw html. the reason we ave the raw html is so that we can use it client side.



237
238
239
240
241
# File 'lib/roda/component.rb', line 237

def tmpl name, dom, remove = true
  cache[:tmpl][name] = { dom: remove ? dom.remove : dom }
  cache[:tmpl][name][:html] = cache[:tmpl][name][:dom].to_html
  cache[:tmpl][name]
end

Instance Method Details

#component_optsObject



302
303
304
# File 'lib/roda/component.rb', line 302

def component_opts
  self.class.component_opts
end

#domObject Also known as: element



281
282
283
284
285
286
287
288
# File 'lib/roda/component.rb', line 281

def dom
  if server?
    # TODO: duplicate cache[:dom] so we don't need to parse all the html again
    @_dom ||= DOM.new(cache[:dom].dom.to_html)
  else
    @_dom ||= DOM.new(Element)
  end
end

#eventsObject



277
278
279
# File 'lib/roda/component.rb', line 277

def events
  @_events ||= Events.new self.class, component_opts, scope
end

#get_value_for(field, data) ⇒ Object



365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
# File 'lib/roda/component.rb', line 365

def get_value_for field, data
  field = (field || '').split '.'

  if field.length > 1
    value = data.is_a?(Hash) ? data.to_obj : data

    field.each_with_index do |f, i|
      # might not have the parent object
      if (value.respond_to?('empty?') ? value.empty? : !value.present?)
        value = ''
        next
      end

      if (i+1) < field.length
        begin
          value = value.send(f)
        rescue
          value = nil
        end
      else
        begin
          value = value.respond_to?(:present) ? value.present("print_#{f}") : value.send(f)
        rescue
          value = nil
        end
      end

    end
  else
    begin
      value = data.respond_to?(:present) ? data.present("print_#{field.first}") : data.send(field.first)
    rescue
      value = nil
    end
  end

  value
end

#inherited(subclass) ⇒ Object



85
86
87
88
89
# File 'lib/roda/component.rb', line 85

def inherited(subclass)
  super
  # We want to set the app for all sub classes
  subclass.set_app app
end

#render_fields(data, options = {}) ⇒ Object



318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
# File 'lib/roda/component.rb', line 318

def render_fields data, options = {}
  data = data.is_a?(Hash) ? data.to_obj : data

  l_dom = options[:dom] || dom

  l_dom.find("[data-if]") do |field_dom|
    value = get_value_for field_dom['data-if'], data

    unless value.present?
      field_dom.remove
    end
  end

  l_dom.find("[data-unless]") do |field_dom|
    value = get_value_for field_dom['data-unless'], data

    if value.present?
      field_dom.remove
    end
  end

  l_dom.find("[data-field]") do |field_dom|
    if field = field_dom['data-field']
      value = get_value_for field, data

      if !value.nil?
        value = value.to_s

        if value != value.upcase && !value.match(Roda::Component::Form::EMAIL)
          field_value = value.titleize
        else
          field_value = value
        end

        field_value = 'No'  if field_value == 'False'
        field_value = 'Yes' if field_value == 'True'

        field_dom.html = field_value
      else
        field_dom.html = ''
      end
    end
  end

  l_dom
end

#tmpl(name) ⇒ Object

Grab the template from the cache, use the nokogiri dom or create a jquery element for server side issue: can’t use the cached dom because duping doesn’t work.



294
295
296
297
298
299
300
# File 'lib/roda/component.rb', line 294

def tmpl name
  if t = cache[:tmpl][name]
    DOM.new t[:html]
  else
    false
  end
end

#trigger(*args) ⇒ Object



306
307
308
# File 'lib/roda/component.rb', line 306

def trigger *args
  events.trigger(*args)
end