Class: Roda::Component::DOM

Inherits:
Object
  • Object
show all
Defined in:
lib/roda/component/dom.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(html) ⇒ DOM

Returns a new instance of DOM.



6
7
8
9
10
11
12
13
14
# File 'lib/roda/component/dom.rb', line 6

def initialize html
  @raw_html = html

  if server?
    @dom = raw_html.is_a?(String) ? Component::HTML(raw_html.dup): raw_html
  else
    @dom = raw_html.is_a?(String) ? Element[raw_html.dup] : raw_html
  end
end

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

#method_missing(method, *args, &block) ⇒ Object

This allows you to use all the nokogiri or opal jquery methods if a global one isn’t set



78
79
80
81
82
83
84
85
# File 'lib/roda/component/dom.rb', line 78

def method_missing method, *args, &block
  # respond_to?(symbol, include_all=false)
  if dom.respond_to? method, true
    dom.send method, *args, &block
  else
    super
  end
end

Instance Attribute Details

#domObject

Returns the value of attribute dom.



4
5
6
# File 'lib/roda/component/dom.rb', line 4

def dom
  @dom
end

#raw_htmlObject

Returns the value of attribute raw_html.



4
5
6
# File 'lib/roda/component/dom.rb', line 4

def raw_html
  @raw_html
end

Instance Method Details

#data(key = false, value = false) ⇒ Object



37
38
39
40
41
42
43
44
45
46
47
# File 'lib/roda/component/dom.rb', line 37

def data key = false, value = false
  d = Hash[node.xpath("@*[starts-with(name(), 'data-')]").map{|a| [a.name, a.value]}]

  if !key
    d
  elsif key && !value
    d[key]
  else
    node["data-#{key}"] = value
  end
end

#find(string, &block) ⇒ Object



16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
# File 'lib/roda/component/dom.rb', line 16

def find string, &block
  if server?
    node = DOM.new dom.css(string)
  else
    node = DOM.new dom.find(string)
  end

  if block
    node.each do |n|
      block.call DOM.new n
    end
  else
    if server?
      node = DOM.new node.first
    end
  end

  node
end

#html(content = false) ⇒ Object



60
61
62
63
64
65
66
67
68
69
70
# File 'lib/roda/component/dom.rb', line 60

def html content = false
  if !content
    if server?
      node.inner_html
    else
      node ? node.html : dom.html
    end
  else
    self.html = content
  end
end

#html=(content) ⇒ Object



50
51
52
53
54
55
56
57
58
# File 'lib/roda/component/dom.rb', line 50

def html= content
  if server?
    node.inner_html = content
  else
    node.html content
  end

  node
end

#nodeObject



72
73
74
# File 'lib/roda/component/dom.rb', line 72

def node
  @node || dom
end