Module: Roda::RodaPlugins::Component::ClassMethods

Defined in:
lib/roda/plugins/component.rb

Instance Method Summary collapse

Instance Method Details

#component_optsObject



155
156
157
# File 'lib/roda/plugins/component.rb', line 155

def component_opts
  opts[:component]
end

#inherited(subclass) ⇒ Object

Copy the assets options into the subclass, duping them as necessary to prevent changes in the subclass affecting the parent class.



149
150
151
152
153
# File 'lib/roda/plugins/component.rb', line 149

def inherited(subclass)
  super
  opts         = component_opts.dup
  opts[:cache] = thread_safe_cache if opts[:cache]
end